#---------------------------------------------------------------------
# 
# A QGIS plugin for creating new postgis layers.
#
# Copyright (C) 2008  Maciej Sieczka
#
# EMAIL: tutey at o2 dot pl
#
#---------------------------------------------------------------------
# 
# licensed under the terms of GNU GPL 2
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
# 
#---------------------------------------------------------------------



#from PyQt4.QtSql import *
from PyQt4.QtGui import *
from PyQt4.QtCore import *

import psycopg
import re

from ui_main_gui import Ui_NewPostgisLayer

import resources

class NewPostgisLayer(QDialog, Ui_NewPostgisLayer):



  def __init__(self, parent, fl):
    QDialog.__init__(self, parent, fl)
    # set up the user interface from Designer
    self.setupUi(self)
    # populate postgres connections list
    self.populateConnections()




  # populate connections combo box from the qgis.conf settings
  def populateConnections(self):
    self.cmbConnection.clear()
    settings = QSettings()
    connGroup = "/PostgreSQL/connections"
    settings.beginGroup(connGroup)
    for key in settings.childGroups():
      self.cmbConnection.addItem(key)
    settings.endGroup()
    if self.cmbConnection.currentText() == '':
      self.btnConnect.setEnabled (False)
    else:
      self.btnConnect.setEnabled (True)



  # populate schemas combo box from the connected database
  def populateSchemas(self):
    self.cmbSchema.clear()
    # fetch schemas names from the connected database
    sql = "SELECT * FROM pg_namespace"
    curs = self.connection.cursor()
    curs.execute(str(sql))
    schemas_raw = str(curs.fetchall())

    #debug
    #print schemas_raw

    # find schemas names in the psycopg output, removing "'" in string type and turning it back into object type
    schemas_lst = eval(str(re.findall('\'\w*\'+', schemas_raw)).replace('\'',''))

    # grep for schemas other than "pg_.*" and "information_schema" (postgres reserved names)
    # using re regexp python module and taking care not to fail on None (empty) list element

    #debug
    #print schemas_lst

    for key in schemas_lst:
      test = re.match('pg_.*', key)
      if test is None:
        if not key == "information_schema":
          #debug
          #print key
          self.cmbSchema.addItem(key)
          schemaName = self.cmbSchema.currentText()
          if not schemaName is None:
            self.lineLayerNam.setEnabled(True)
            self.lineIndexCol.setEnabled(True)
            self.lineGeomCol.setEnabled(True)
            self.cmbGeomTyp.setEnabled(True)
            self.spinSridNum.setEnabled(True)

  # extract parameterers for psycopg from the selected connection
  def on_btnConnect_clicked(self):
    connName = self.cmbConnection.currentText()
    settings = QSettings()
    host = settings.value("/PostgreSQL/connections/"+connName+"/host").toString()
    port = settings.value("/PostgreSQL/connections/"+connName+"/port").toString()
    dbname = settings.value("/PostgreSQL/connections/"+connName+"/database").toString()
    user = settings.value("/PostgreSQL/connections/"+connName+"/username").toString()
    password = settings.value("/PostgreSQL/connections/"+connName+"/password").toString()
    connection = str("host="+host+" port="+port+" dbname="+dbname+" user="+user+" password="+password)

    try:
        self.connection = psycopg.connect(str(connection))
    except Exception, e:
        QMessageBox.critical(self, "Error", "Cannot establish connection '" +str(connName)+ "'.\n\nThe connector reported:\n\n"\
        +str(e), QMessageBox.Ok)
    else:
        #debug
        #print connection
        self.populateSchemas()
        self.cmbSchema.setEnabled(True)


  #check if all fields in th GUI are filled
  def checkFilled(self):
    if (self.lineLayerNam.text() \
    and self.lineIndexCol.text() \
    and self.lineGeomCol.text() \
    and self.spinSridNum.text() ):
      self.btnOk.setEnabled(True)
    else:
      self.btnOk.setEnabled(False)

  def on_lineLayerNam_textChanged(self):
    self.checkFilled()

  def on_lineIndexCol_textChanged(self):
    self.checkFilled()

  def on_lineGeomCol_textChanged(self):
    self.checkFilled()

  def on_spinSridNum_textChanged(self):
    self.checkFilled()


  #if all OK create the table
  def on_btnOk_clicked(self):
    schemaName = self.cmbSchema.currentText()
    newLayerName = self.lineLayerNam.text()
    indexColName = self.lineIndexCol.text()
    geomColName = self.lineGeomCol.text()
    geomTypName = self.cmbGeomTyp.currentText()
    sridNum = self.spinSridNum.text()
    #debug
    #print schemaName, newLayerName, indexColName, geomColName, geomTypName, sridNum

    try:
      curs = self.connection.cursor()
      #debug
      #print (str( "CREATE TABLE " +schemaName+ "." +newLayerName+ " (" +indexColName+ " serial primary key)" ))
      curs.execute (str( "CREATE TABLE " +schemaName+ "." +newLayerName+ " (" +indexColName+ " serial primary key)" ))

      #debug
      #print (str( "select AddGeometryColumn('" +schemaName+ "', '" +newLayerName+ \
      #"', '" +geomColName+ "', "+sridNum+ ", '" +geomTypName+ "', 2)" ))

      curs.execute (str( "select AddGeometryColumn('" +schemaName+ "', '" +newLayerName+ \
      "', '" +geomColName+ "', "+sridNum+ ", '" +geomTypName+ "', 2)" ))

      # create the GIST index
      #debug
      #print (str ("create index sidx_" +newLayerName+ " on " + \
      #schemaName+ "." +newLayerName+ " USING GIST("+geomColName+" GIST_GEOMETRY_OPS)") )

      curs.execute (str ("create index sidx_" +newLayerName+ " on " + \
      schemaName+ "." +newLayerName+ " USING GIST("+geomColName+" GIST_GEOMETRY_OPS)") )
      # commit operations
      self.connection.commit()

    except Exception, e:
      QMessageBox.critical(self, "Error", "Cannot cannot create the new PostGIS table.\n\nThe connector reported:\n\n"\
      +str(e) ,  QMessageBox.Ok)
      self.connection.rollback()
    else:
      QMessageBox.warning(self, "Success", "Layer '" +str(newLayerName)+  "' has been successfully created in schema '"\
      +str(schemaName)+ "'." ,  QMessageBox.Ok)
